@@ -79,8 +79,8 @@ struct trap_sink *sinks = NULL;
 
 extern struct timeval starttime;
 
-oid             objid_enterprisetrap[] = { NOTIFICATION_MIB };
-oid             trap_version_id[] = { SYSTEM_MIB };
+oid             objid_enterprisetrap[] = { NETSNMP_NOTIFICATION_MIB };
+oid             trap_version_id[] = { NETSNMP_SYSTEM_MIB };
 int             enterprisetrap_len;
 int             trap_version_id_len;
 
@@ -107,7 +107,7 @@ oid             agentaddr_oid[] = { SNMPV2_COMM_OBJS_PREFIX, 3, 0 };
 size_t          agentaddr_oid_len;
 oid             community_oid[] = { SNMPV2_COMM_OBJS_PREFIX, 4, 0 };
 size_t          community_oid_len;
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
 char           *snmp_trapcommunity = NULL;
 #endif
 
@@ -224,26 +224,15 @@ remove_trap_session(netsnmp_session * ss)
     return 0;
 }
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
-int
-create_trap_session(char *sink, u_short sinkport,
-                    char *com, int version, int pdutype)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
+static int
+create_trap_session2(const char *sink, const char* sinkport,
+		     char *com, int version, int pdutype)
 {
+    netsnmp_transport *t;
     netsnmp_session session, *sesp;
-    char           *peername = NULL;
-    int             len;
-
-    len = strlen(sink) + 4 + 32;
-    if ((peername = malloc(len)) == NULL) {
-        return 0;
-    } else if (NULL != strchr(sink,':')) {
-        snprintf(peername, len, "%s", sink);
-    } else {
-        snprintf(peername, len, "udp:%s:%hu", sink, sinkport);
-    }
 
     memset(&session, 0, sizeof(netsnmp_session));
-    session.peername = peername;
     session.version = version;
     if (com) {
         session.community = (u_char *) com;
@@ -265,44 +254,62 @@ create_trap_session(char *sink, u_short sinkport,
                                        NETSNMP_DS_LIB_CLIENT_ADDR)) && 
         ((0 == strcmp("localhost",sink)) || (0 == strcmp("127.0.0.1",sink))))
         session.localname = "localhost";
-    sesp = snmp_open(&session);
-    free(peername);
 
-    if (sesp) {
-        return add_trap_session(sesp, pdutype,
-                                (pdutype == SNMP_MSG_INFORM), version);
+    t = netsnmp_tdomain_transport_full("snmptrap", sink, 0, NULL, sinkport);
+    if (t != NULL) {
+	sesp = snmp_add(&session, t, NULL, NULL);
+
+	if (sesp) {
+	    return add_trap_session(sesp, pdutype,
+				    (pdutype == SNMP_MSG_INFORM), version);
+	}
     }
-
     /*
      * diagnose snmp_open errors with the input netsnmp_session pointer 
      */
     snmp_sess_perror("snmpd: create_trap_session", &session);
     return 0;
 }
+
+int
+create_trap_session(char *sink, u_short sinkport,
+		    char *com, int version, int pdutype)
+{
+    char buf[sizeof(sinkport) * 3 + 2];
+    if (sinkport != 0) {
+	sprintf(buf, ":%hu", sinkport);
+	snmp_log(LOG_NOTICE,
+		 "Using a separate port number is deprecated, please correct "
+		 "the sink specification instead");
+    }
+    return create_trap_session2(sink, sinkport ? buf : NULL, com, version,
+				pdutype);
+}
+
 #endif /* support for community based SNMP */
 
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
 static int
-create_v1_trap_session(char *sink, u_short sinkport, char *com)
+create_v1_trap_session(char *sink, const char *sinkport, char *com)
 {
-    return create_trap_session(sink, sinkport, com,
-                               SNMP_VERSION_1, SNMP_MSG_TRAP);
+    return create_trap_session2(sink, sinkport, com,
+				SNMP_VERSION_1, SNMP_MSG_TRAP);
 }
 #endif
 
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
 static int
-create_v2_trap_session(char *sink, u_short sinkport, char *com)
+create_v2_trap_session(const char *sink, const char *sinkport, char *com)
 {
-    return create_trap_session(sink, sinkport, com,
-                               SNMP_VERSION_2c, SNMP_MSG_TRAP2);
+    return create_trap_session2(sink, sinkport, com,
+				SNMP_VERSION_2c, SNMP_MSG_TRAP2);
 }
 
 static int
-create_v2_inform_session(char *sink, u_short sinkport, char *com)
+create_v2_inform_session(const char *sink, const char *sinkport, char *com)
 {
-    return create_trap_session(sink, sinkport, com,
-                               SNMP_VERSION_2c, SNMP_MSG_INFORM);
+    return create_trap_session2(sink, sinkport, com,
+				SNMP_VERSION_2c, SNMP_MSG_INFORM);
 }
 #endif
 
@@ -774,14 +781,14 @@ netsnmp_send_traps(int trap, int specific,
      *   providing an appropriately formatted PDU in each case
      */
     for (sink = sinks; sink; sink = sink->next) {
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
         if (sink->version == SNMP_VERSION_1) {
             send_trap_to_sess(sink->sesp, template_v1pdu);
         } else {
 #endif
             template_v2pdu->command = sink->pdutype;
             send_trap_to_sess(sink->sesp, template_v2pdu);
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
         }
 #endif
     }
@@ -862,7 +869,7 @@ send_trap_to_sess(netsnmp_session * sess, netsnmp_pdu *template_pdu)
     DEBUGMSGTL(("trap", "sending trap type=%d, version=%d\n",
                 template_pdu->command, sess->version));
 
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     if (sess->version == SNMP_VERSION_1 &&
         (template_pdu->command != SNMP_MSG_TRAP))
         return;                 /* Skip v1 sinks for v2 only traps */
@@ -910,10 +917,10 @@ send_trap_vars(int trap, int specific, netsnmp_variable_list * vars)
  *
  * This function eventually calls send_enterprise_trap_vars.  If the
  * trap type is not set to SNMP_TRAP_ENTERPRISESPECIFIC the enterprise 
- * and enterprise_length paramater is set to the pre defined SYSTEM_MIB 
+ * and enterprise_length paramater is set to the pre defined NETSNMP_SYSTEM_MIB 
  * oid and length respectively.  If the trap type is set to 
  * SNMP_TRAP_ENTERPRISESPECIFIC the enterprise and enterprise_length 
- * parameters are set to the pre-defined NOTIFICATION_MIB oid and length 
+ * parameters are set to the pre-defined NETSNMP_NOTIFICATION_MIB oid and length 
  * respectively.
  *
  * @param trap is the generic trap type.
@@ -940,10 +947,10 @@ send_easy_trap(int trap, int specific)
  *
  * This function eventually calls send_enterprise_trap_vars.  If the
  * trap type is not set to SNMP_TRAP_ENTERPRISESPECIFIC the enterprise 
- * and enterprise_length paramater is set to the pre defined SYSTEM_MIB 
+ * and enterprise_length paramater is set to the pre defined NETSNMP_SYSTEM_MIB 
  * oid and length respectively.  If the trap type is set to 
  * SNMP_TRAP_ENTERPRISESPECIFIC the enterprise and enterprise_length 
- * parameters are set to the pre-defined NOTIFICATION_MIB oid and length 
+ * parameters are set to the pre-defined NETSNMP_NOTIFICATION_MIB oid and length 
  * respectively.
  *
  * @param vars is used to supply list of variable bindings to form an SNMPv2 
@@ -1023,13 +1030,12 @@ snmpd_parse_config_authtrap(const char *token, char *cptr)
     }
 }
 
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
 void
 snmpd_parse_config_trapsink(const char *token, char *cptr)
 {
     char            tmpbuf[1024];
     char           *sp, *cp, *pp = NULL;
-    int             sinkport;
     char            *st;
 
     if (!snmp_trapcommunity)
@@ -1038,17 +1044,9 @@ snmpd_parse_config_trapsink(const char *token, char *cptr)
     cp = strtok_r(NULL, " \t\n", &st);
     if (cp)
         pp = strtok_r(NULL, " \t\n", &st);
-    if (cp && pp) {
-        sinkport = atoi(pp);
-        if ((sinkport < 1) || (sinkport > 0xffff)) {
-            config_perror("trapsink port out of range");
-            sinkport = SNMP_TRAP_PORT;
-        }
-    } else {
-        sinkport = SNMP_TRAP_PORT;
-    }
-    if (create_v1_trap_session(sp, (u_short)sinkport,
-                               cp ? cp : snmp_trapcommunity) == 0) {
+    if (pp)
+	config_pwarn("The separate port argument to trapsink is deprecated");
+    if (create_v1_trap_session(sp, pp, cp ? cp : snmp_trapcommunity) == 0) {
         snprintf(tmpbuf, sizeof(tmpbuf), "cannot create trapsink: %s", cptr);
         tmpbuf[sizeof(tmpbuf)-1] = '\0';
         config_perror(tmpbuf);
@@ -1056,7 +1054,7 @@ snmpd_parse_config_trapsink(const char *token, char *cptr)
 }
 #endif
 
-#ifndef DISABLE_SNMPV2C
+#ifndef NETSNMP_DISABLE_SNMPV2C
 void
 snmpd_parse_config_trap2sink(const char *word, char *cptr)
 {
@@ -1071,17 +1069,9 @@ snmpd_parse_config_trap2sink(const char *word, char *cptr)
     cp = strtok_r(NULL, " \t\n", &st);
     if (cp)
         pp = strtok_r(NULL, " \t\n", &st);
-    if (cp && pp) {
-        sinkport = atoi(pp);
-        if ((sinkport < 1) || (sinkport > 0xffff)) {
-            config_perror("trapsink port out of range");
-            sinkport = SNMP_TRAP_PORT;
-        }
-    } else {
-        sinkport = SNMP_TRAP_PORT;
-    }
-    if (create_v2_trap_session(sp, (u_short)sinkport,
-                               cp ? cp : snmp_trapcommunity) == 0) {
+    if (pp)
+	config_pwarn("The separate port argument to trapsink2 is deprecated");
+    if (create_v2_trap_session(sp, pp, cp ? cp : snmp_trapcommunity) == 0) {
         snprintf(tmpbuf, sizeof(tmpbuf), "cannot create trap2sink: %s", cptr);
         tmpbuf[sizeof(tmpbuf)-1] = '\0';
         config_perror(tmpbuf);
@@ -1102,17 +1092,9 @@ snmpd_parse_config_informsink(const char *word, char *cptr)
     cp = strtok_r(NULL, " \t\n", &st);
     if (cp)
         pp = strtok_r(NULL, " \t\n", &st);
-    if (cp && pp) {
-        sinkport = atoi(pp);
-        if ((sinkport < 1) || (sinkport > 0xffff)) {
-            config_perror("trapsink port out of range");
-            sinkport = SNMP_TRAP_PORT;
-        }
-    } else {
-        sinkport = SNMP_TRAP_PORT;
-    }
-    if (create_v2_inform_session(sp, (u_short)sinkport,
-                                 cp ? cp : snmp_trapcommunity) == 0) {
+    if (pp)
+	config_pwarn("The separate port argument to informsink is deprecated");
+    if (create_v2_inform_session(sp, pp, cp ? cp : snmp_trapcommunity) == 0) {
         snprintf(tmpbuf, sizeof(tmpbuf), "cannot create informsink: %s", cptr);
         tmpbuf[sizeof(tmpbuf)-1] = '\0';
         config_perror(tmpbuf);
@@ -1153,6 +1135,7 @@ snmpd_parse_config_trapsess(const char *word, char *cptr)
     char           *argv[MAX_ARGS], *cp = cptr, tmp[SPRINT_MAX_LEN];
     int             argn, arg;
     netsnmp_session session, *ss;
+    size_t          len;
 
     /*
      * inform or trap?  default to trap 
@@ -1182,19 +1165,31 @@ snmpd_parse_config_trapsess(const char *word, char *cptr)
         return;
     }
 
-#ifndef DISABLE_SNMPV1
+    /*
+     * If this is an SNMPv3 TRAP session, then the agent is
+     *   the authoritative engine, so set the engineID accordingly
+     */
+    if (ss->version == SNMP_VERSION_3 &&
+        traptype != SNMP_MSG_INFORM   &&
+        ss->securityEngineIDLen == 0) {
+            len = snmpv3_get_engineID( tmp, sizeof(tmp));
+            memdup(&ss->securityEngineID, tmp, len);
+            ss->securityEngineIDLen = len;
+    }
+
+#ifndef NETSNMP_DISABLE_SNMPV1
     if (ss->version == SNMP_VERSION_1) {
         add_trap_session(ss, SNMP_MSG_TRAP, 0, SNMP_VERSION_1);
     } else {
 #endif
         add_trap_session(ss, traptype, (traptype == SNMP_MSG_INFORM),
                          ss->version);
-#ifndef DISABLE_SNMPV1
+#ifndef NETSNMP_DISABLE_SNMPV1
     }
 #endif
 }
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
 void
 snmpd_parse_config_trapcommunity(const char *word, char *cptr)
 {
