@@ -145,7 +145,7 @@
 
 #if !defined (WIN32) && !defined (cygwin)
 
-#if !defined(CAN_USE_SYSCTL) || !defined(IPCTL_STATS)
+#if !defined(NETSNMP_CAN_USE_SYSCTL) || !defined(IPCTL_STATS)
 #ifndef solaris2
 
 #if defined(freebsd2) || defined(hpux11) || defined(linux)
@@ -293,6 +293,7 @@ var_ipAddrEntry(struct variable *vp,
     *var_len = sizeof(long_return);
     switch (vp->magic) {
     case IPADADDR:
+	*var_len = sizeof(uint32_t);
 #ifdef hpux11
         long_return = lowin_ifaddr.Addr;
         return (u_char *) & long_return;
@@ -308,6 +309,7 @@ var_ipAddrEntry(struct variable *vp,
         long_return = lowinterface;
         return (u_char *) & long_return;
     case IPADNETMASK:
+	*var_len = sizeof(uint32_t);
 #ifdef hpux11
         long_return = lowin_ifaddr.NetMask;
         return (u_char *) & long_return;
@@ -341,7 +343,7 @@ var_ipAddrEntry(struct variable *vp,
 #ifdef hpux11
         long_return = lowin_ifaddr.ReasmMaxSize;
         return (u_char *) & long_return;
-#elif defined(NO_DUMMY_VALUES)
+#elif defined(NETSNMP_NO_DUMMY_VALUES)
         return NULL;
 #else
         long_return = -1;
@@ -382,8 +384,11 @@ Address_Scan_Next(Index, Retin_ifaddr)
         /*
          *      Get the "in_ifaddr" structure
          */
-        klookup((unsigned long) in_ifaddraddr, (char *) &in_ifaddr,
-                sizeof in_ifaddr);
+        if (!NETSNMP_KLOOKUP(in_ifaddraddr, (char *) &in_ifaddr, sizeof in_ifaddr)) {
+            DEBUGMSGTL(("mibII/ip:Address_Scan_Next", "klookup failed\n"));
+            return 0;
+        }
+
         in_ifaddraddr = in_ifaddr.ia_next;
 
         if (Retin_ifaddr)
@@ -396,8 +401,10 @@ Address_Scan_Next(Index, Retin_ifaddr)
 
         auto_nlist(IFNET_SYMBOL, (char *) &ifnetaddr, sizeof(ifnetaddr));
         while (ifnetaddr && ifnetaddr != in_ifaddr.ia_ifp) {
-            klookup((unsigned long) ifnetaddr, (char *) &ifnet,
-                    sizeof ifnet);
+            if (!NETSNMP_KLOOKUP(ifnetaddr, (char *) &ifnet, sizeof ifnet)) {
+                DEBUGMSGTL(("mibII/ip:Address_Scan_Next", "klookup failed\n"));
+                return 0;
+            }
             ifnetaddr = ifnet.if_next;
             iindex++;
         }
@@ -694,9 +701,16 @@ var_ipAddrEntry(struct variable * vp,
         ipaddr_return = Lowentry.ipAdEntAddr;
         return (u_char *) & ipaddr_return;
     case IPADIFINDEX:
+#ifdef NETSNMP_INCLUDE_IFTABLE_REWRITES
+        Lowentry.ipAdEntIfIndex.o_bytes[Lowentry.ipAdEntIfIndex.o_length] = '\0';
         long_return =
-            Interface_Index_By_Name(Lowentry.ipAdEntIfIndex.o_bytes,
-                                    Lowentry.ipAdEntIfIndex.o_length);
+            netsnmp_access_interface_index_find(Lowentry.
+                                                ipAdEntIfIndex.o_bytes);
+#else
+        long_return =
+           Interface_Index_By_Name(Lowentry.ipAdEntIfIndex.o_bytes,
+                                   Lowentry.ipAdEntIfIndex.o_length);
+#endif
         return (u_char *) & long_return;
     case IPADNETMASK:
 	*var_len = sizeof(uint32_t);
@@ -726,7 +740,7 @@ var_ipAddrEntry(struct variable * vp,
 	 *********************/
 
 
-#else                           /* CAN_USE_SYSCTL && IPCTL_STATS */
+#else                           /* NETSNMP_CAN_USE_SYSCTL && IPCTL_STATS */
 
 /*
  * Ideally, this would be combined with the code in interfaces.c.
@@ -912,6 +926,7 @@ var_ipAddrEntry(struct variable *vp,
     *var_len = sizeof(long_return);
     switch (vp->magic) {
     case IPADADDR:
+	*var_len = sizeof(uint32_t);
         long_return = ifs[i].addr.s_addr;
         return (u_char *) & long_return;
 
@@ -920,6 +935,7 @@ var_ipAddrEntry(struct variable *vp,
         return (u_char *) & long_return;
 
     case IPADNETMASK:
+	*var_len = sizeof(uint32_t);
         long_return = ifs[i].mask.s_addr;
         return (u_char *) & long_return;
 
@@ -928,7 +944,7 @@ var_ipAddrEntry(struct variable *vp,
         return (u_char *) & long_return;
 
     case IPADREASMMAX:
-#if NO_DUMMY_VALUES
+#if NETSNMP_NO_DUMMY_VALUES
         return NULL;
 #else
         long_return = -1;
@@ -942,7 +958,7 @@ var_ipAddrEntry(struct variable *vp,
     return NULL;
 }
 
-#endif                          /* CAN_USE_SYSCTL && IPCTL_STATS */
+#endif                          /* NETSNMP_CAN_USE_SYSCTL && IPCTL_STATS */
 
 #else                           /* WIN32 cygwin */
 #include <iphlpapi.h>
@@ -1021,6 +1037,7 @@ var_ipAddrEntry(struct variable *vp,
     *var_len = sizeof(long_return);
     switch (vp->magic) {
     case IPADADDR:
+	*var_len = sizeof(uint32_t);
         long_return = pIpAddrTable->table[i].dwAddr;
         return (u_char *) & long_return;
 
@@ -1030,6 +1047,7 @@ var_ipAddrEntry(struct variable *vp,
         return (u_char *) & long_return;
 
     case IPADNETMASK:
+	*var_len = sizeof(uint32_t);
         long_return = pIpAddrTable->table[i].dwMask;
         free(pIpAddrTable);
         return (u_char *) & long_return;
