@@ -308,7 +308,7 @@ netsnmp_unix_transport(struct sockaddr_un *addr, int local)
     t->flags = NETSNMP_TRANSPORT_FLAG_STREAM;
 
     if (local) {
-        t->local = malloc(strlen(addr->sun_path));
+      t->local = (u_char *)malloc(strlen(addr->sun_path));
         if (t->local == NULL) {
             netsnmp_transport_free(t);
             return NULL;
@@ -358,7 +358,7 @@ netsnmp_unix_transport(struct sockaddr_un *addr, int local)
         }
 
     } else {
-        t->remote = malloc(strlen(addr->sun_path));
+      t->remote = (u_char *)malloc(strlen(addr->sun_path));
         if (t->remote == NULL) {
             netsnmp_transport_free(t);
             return NULL;
@@ -405,17 +405,24 @@ netsnmp_unix_transport(struct sockaddr_un *addr, int local)
 }
 
 netsnmp_transport *
-netsnmp_unix_create_tstring(const char *string, int local)
+netsnmp_unix_create_tstring(const char *string, int local,
+			    const char *default_target)
 {
     struct sockaddr_un addr;
 
-    if ((string != NULL) && (strlen(string) < sizeof(addr.sun_path))) {
+    if (string && *string != '\0') {
+    } else if (default_target && *default_target != '\0') {
+      string = default_target;
+    }
+
+    if ((string != NULL && *string != '\0') &&
+	(strlen(string) < sizeof(addr.sun_path))) {
         addr.sun_family = AF_UNIX;
         memset(addr.sun_path, 0, sizeof(addr.sun_path));
         strncpy(addr.sun_path, string, sizeof(addr.sun_path) - 1);
         return netsnmp_unix_transport(&addr, local);
     } else {
-        if (string != NULL) {
+        if (string != NULL && *string != '\0') {
             snmp_log(LOG_ERR, "Path too long for Unix domain transport\n");
         }
         return NULL;
@@ -432,7 +439,7 @@ netsnmp_unix_create_ostring(const u_char * o, size_t o_len, int local)
     if (o_len > 0 && o_len < (sizeof(addr.sun_path) - 1)) {
         addr.sun_family = AF_UNIX;
         memset(addr.sun_path, 0, sizeof(addr.sun_path));
-        strncpy(addr.sun_path, o, o_len);
+        strncpy(addr.sun_path, (char *)o, o_len);
         return netsnmp_unix_transport(&addr, local);
     } else {
         if (o_len > 0) {
@@ -449,16 +456,16 @@ netsnmp_unix_ctor(void)
 {
     unixDomain.name = netsnmp_UnixDomain;
     unixDomain.name_length = sizeof(netsnmp_UnixDomain) / sizeof(oid);
-    unixDomain.prefix = calloc(2, sizeof(char *));
+    unixDomain.prefix = (const char**)calloc(2, sizeof(char *));
     unixDomain.prefix[0] = "unix";
 
-    unixDomain.f_create_from_tstring = netsnmp_unix_create_tstring;
+    unixDomain.f_create_from_tstring_new = netsnmp_unix_create_tstring;
     unixDomain.f_create_from_ostring = netsnmp_unix_create_ostring;
 
     netsnmp_tdomain_register(&unixDomain);
 }
 
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
 /* support for SNMPv1 and SNMPv2c on unix domain*/
 
 #define EXAMPLE_COMMUNITY "COMMUNITY"
@@ -640,7 +647,7 @@ netsnmp_unix_com2SecList_free(void)
 void
 netsnmp_unix_agent_config_tokens_register(void)
 {
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
     register_app_config_handler("com2secunix", netsnmp_unix_parse_security,
                                 netsnmp_unix_com2SecList_free,
                                 "[-Cn CONTEXT] secName sockpath community");
